@@ -1,9 +1,13 @@
+from __future__ import annotations
+
+import logging
 from abc import ABC
-from typing import Dict, List, Optional, Tuple, Union
+from typing import Any
 
 import pandas as pd
 import sqlalchemy
 from airflow.hooks.dbapi import DbApiHook
+from pandas.io.sql import SQLDatabase
 from sqlalchemy import column, insert, select
 from sqlalchemy.sql import ClauseElement
 from sqlalchemy.sql.elements import ColumnClause
@@ -11,13 +15,14 @@ from sqlalchemy.sql.schema import Table as SqlaTable
 
 from astro.constants import (
     DEFAULT_CHUNK_SIZE,
+    ColumnCapitalization,
     ExportExistsStrategy,
     LoadExistStrategy,
     MergeConflictStrategy,
 )
-from astro.exceptions import NonExistentTableException
-from astro.files import File
-from astro.settings import SCHEMA
+from astro.exceptions import DatabaseCustomError, NonExistentTableException
+from astro.files import File, resolve_file_path_pattern
+from astro.settings import LOAD_TABLE_AUTODETECT_ROWS_COUNT, SCHEMA
 from astro.sql.table import Metadata, Table
 
 
@@ -37,15 +42,16 @@ class BaseDatabase(ABC):
     _create_schema_statement: str = "CREATE SCHEMA IF NOT EXISTS {}"
     _drop_table_statement: str = "DROP TABLE IF EXISTS {}"
     _create_table_statement: str = "CREATE TABLE IF NOT EXISTS {} AS {}"
-
     # Used to normalize the ndjson when appending fields in nested fields.
     # Example -
     #   ndjson - {'a': {'b': 'val'}}
     #   the col names generated is 'a.b'. char '.' maybe an illegal char in some db's col name.
     # Contains the illegal char and there replacement, where the value in
     # illegal_column_name_chars[0] will be replaced by value in illegal_column_name_chars_replacement[0]
-    illegal_column_name_chars: List[str] = []
-    illegal_column_name_chars_replacement: List[str] = []
+    illegal_column_name_chars: list[str] = []
+    illegal_column_name_chars_replacement: list[str] = []
+    NATIVE_LOAD_EXCEPTIONS: Any = DatabaseCustomError
+    DEFAULT_SCHEMA = SCHEMA
 
     def __init__(self, conn_id: str):
         self.conn_id = conn_id
@@ -74,8 +80,8 @@ class BaseDatabase(ABC):
 
     def run_sql(
         self,
-        sql_statement: Union[str, ClauseElement],
-        parameters: Optional[dict] = None,
+        sql_statement: str | ClauseElement,
+        parameters: dict | None = None,
     ):
         """
         Return the results to running a SQL statement.
@@ -109,7 +115,7 @@ class BaseDatabase(ABC):
     # Table metadata
     # ---------------------------------------------------------
     @staticmethod
-    def get_merge_initialization_query(parameters: Tuple) -> str:
+    def get_merge_initialization_query(parameters: tuple) -> str:
         """
         Handles database-specific logic to handle constraints, keeping
         it agnostic to database.
@@ -158,27 +164,89 @@ class BaseDatabase(ABC):
         if table.metadata and table.metadata.is_empty() and self.default_metadata:
             table.metadata = self.default_metadata
         if not table.metadata.schema:
-            table.metadata.schema = SCHEMA
+            table.metadata.schema = self.DEFAULT_SCHEMA
         return table
 
     # ---------------------------------------------------------
     # Table creation & deletion methods
     # ---------------------------------------------------------
-    def create_table(self, table: Table) -> None:
+    def create_table_using_columns(self, table: Table) -> None:
         """
-        Create a SQL table. For this method to work, the table instance must contain columns.
+        Create a SQL table using the table columns.
 
         :param table: The table to be created.
         """
+        if not table.columns:
+            raise ValueError("To use this method, table.columns must be defined")
         metadata = table.sqlalchemy_metadata
         sqlalchemy_table = sqlalchemy.Table(table.name, metadata, *table.columns)
         metadata.create_all(self.sqlalchemy_engine, tables=[sqlalchemy_table])
 
+    def create_table_using_schema_autodetection(
+        self,
+        table: Table,
+        file: File | None = None,
+        dataframe: pd.DataFrame | None = None,
+        columns_names_capitalization: ColumnCapitalization = "lower",  # skipcq
+    ) -> None:
+        """
+        Create a SQL table, automatically inferring the schema using the given file.
+
+        :param table: The table to be created.
+        :param file: File used to infer the new table columns.
+        :param dataframe: Dataframe used to infer the new table columns if there is no file
+        :param columns_names_capitalization: determines whether to convert all columns to lowercase/uppercase
+            in the resulting dataframe
+        """
+        if file is None:
+            if dataframe is None:
+                raise ValueError(
+                    "File or Dataframe is required for creating table using schema autodetection"
+                )
+            source_dataframe = dataframe
+        else:
+            source_dataframe = file.export_to_dataframe(
+                nrows=LOAD_TABLE_AUTODETECT_ROWS_COUNT
+            )
+
+        db = SQLDatabase(engine=self.sqlalchemy_engine)
+        db.prep_table(
+            source_dataframe,
+            table.name.lower(),
+            schema=table.metadata.schema,
+            if_exists="replace",
+            index=False,
+        )
+
+    def create_table(
+        self,
+        table: Table,
+        file: File | None = None,
+        dataframe: pd.DataFrame | None = None,
+        columns_names_capitalization: ColumnCapitalization = "original",
+    ) -> None:
+        """
+        Create a table either using its explicitly defined columns or inferring
+        it's columns from a given file.
+
+        :param table: The table to be created
+        :param file: (optional) File used to infer the table columns.
+        :param dataframe: (optional) Dataframe used to infer the new table columns if there is no file
+        :param columns_names_capitalization: determines whether to convert all columns to lowercase/uppercase
+            in the resulting dataframe
+        """
+        if table.columns:
+            self.create_table_using_columns(table)
+        else:
+            self.create_table_using_schema_autodetection(
+                table, file, dataframe, columns_names_capitalization
+            )
+
     def create_table_from_select_statement(
         self,
         statement: str,
         target_table: Table,
-        parameters: Optional[dict] = None,
+        parameters: dict | None = None,
     ) -> None:
         """
         Export the result rows of a query statement into another table.
@@ -206,29 +274,115 @@ class BaseDatabase(ABC):
     # ---------------------------------------------------------
     # Table load methods
     # ---------------------------------------------------------
+
     def load_file_to_table(
+        self,
+        input_file: File,
+        output_table: Table,
+        normalize_config: dict | None = None,
+        if_exists: LoadExistStrategy = "replace",
+        chunk_size: int = DEFAULT_CHUNK_SIZE,
+        use_native_support: bool = True,
+        native_support_kwargs: dict | None = None,
+        columns_names_capitalization: ColumnCapitalization = "original",
+        enable_native_fallback: bool | None = True,
+        **kwargs,
+    ):
+        """
+        Load content of multiple files in output_table.
+        Multiple files are sourced from the file path, which can also be path pattern.
+
+        :param input_file: File path and conn_id for object stores
+        :param output_table: Table to create
+        :param if_exists: Overwrite file if exists
+        :param chunk_size: Specify the number of records in each batch to be written at a time
+        :param use_native_support: Use native support for data transfer if available on the destination
+        :param normalize_config: pandas json_normalize params config
+        :param native_support_kwargs: kwargs to be used by method involved in native support flow
+        :param columns_names_capitalization: determines whether to convert all columns to lowercase/uppercase
+            in the resulting dataframe
+        :param enable_native_fallback: Use enable_native_fallback=True to fall back to default transfer
+        """
+        normalize_config = normalize_config or {}
+        input_files = resolve_file_path_pattern(
+            input_file.path,
+            input_file.conn_id,
+            normalize_config=normalize_config,
+        )
+        self.create_schema_if_needed(output_table.metadata.schema)
+        if if_exists == "replace" or not self.table_exists(output_table):
+            self.drop_table(output_table)
+            self.create_table(
+                output_table,
+                input_files[0],
+                columns_names_capitalization=columns_names_capitalization,
+            )
+            if_exists = "append"
+
+        # TODO: many native transfers support the input_file.path - it may be better
+        # to use the native support to loading multiple files as opposed to iterating
+        # here
+        for file in input_files:
+            if use_native_support and self.is_native_load_file_available(
+                source_file=file, target_table=output_table
+            ):
+                self.load_file_to_table_natively_with_fallback(
+                    source_file=file,
+                    target_table=output_table,
+                    if_exists=if_exists,
+                    native_support_kwargs=native_support_kwargs,
+                    enable_native_fallback=enable_native_fallback,
+                    chunk_size=chunk_size,
+                )
+            else:
+                self.load_pandas_dataframe_to_table(
+                    file.export_to_dataframe(),
+                    output_table,
+                    chunk_size=chunk_size,
+                    if_exists="append",  # We've already created a new table in this case
+                )
+
+    def load_file_to_table_natively_with_fallback(
         self,
         source_file: File,
         target_table: Table,
         if_exists: LoadExistStrategy = "replace",
+        native_support_kwargs: dict | None = None,
+        enable_native_fallback: bool | None = True,
         chunk_size: int = DEFAULT_CHUNK_SIZE,
-    ) -> None:
+        **kwargs,
+    ):
         """
-        Load the content of the source file to the target database table.
-        If the table already exists, append or replace the content, depending on the value of `if_exists`.
+        Load content of a file in output_table.
 
-        :param source_file: Local or remote filepath (e.g. a File("/tmp/sample_data.csv"))
-        :param target_table: Table in which the file will be loaded
-        :param if_exists: Strategy to be used in case the target table already exists
-        :param chunk_size: Specify the number of rows in each batch to be written at a time.
+        :param source_file: File path and conn_id for object stores
+        :param target_table: Table to create
+        :param if_exists: Overwrite file if exists
+        :param chunk_size: Specify the number of records in each batch to be written at a time
+        :param native_support_kwargs: kwargs to be used by method involved in native support flow
+        :param enable_native_fallback: Use enable_native_fallback=True to fall back to default transfer.
         """
-        dataframe = source_file.export_to_dataframe()
-        self.load_pandas_dataframe_to_table(
-            dataframe,
-            target_table,
-            if_exists,
-            chunk_size,
-        )
+        try:
+            self.load_file_to_table_natively(
+                source_file=source_file,
+                target_table=target_table,
+                if_exists=if_exists,
+                native_support_kwargs=native_support_kwargs,
+                **kwargs,
+            )
+        # Catching NATIVE_LOAD_EXCEPTIONS for fallback
+        except self.NATIVE_LOAD_EXCEPTIONS:  # skipcq: PYL-W0703
+            logging.warning(
+                "Loading files failed with Native Support. Falling back to Pandas-based load",
+                exc_info=True,
+            )
+            if enable_native_fallback:
+                self.load_pandas_dataframe_to_table(
+                    source_file.export_to_dataframe(),
+                    target_table=target_table,
+                    chunk_size=chunk_size,
+                    if_exists=if_exists,
+                )
 
     def load_pandas_dataframe_to_table(
         self,
@@ -246,7 +400,6 @@ class BaseDatabase(ABC):
         :param if_exists: Strategy to be used in case the target table already exists.
         :param chunk_size: Specify the number of rows in each batch to be written at a time.
         """
-        self.create_schema_if_needed(target_table.metadata.schema)
         source_dataframe.to_sql(
             self.get_table_qualified_name(target_table),
             con=self.sqlalchemy_engine,
@@ -260,7 +413,7 @@ class BaseDatabase(ABC):
         self,
         source_table: Table,
         target_table: Table,
-        source_to_target_columns_map: Dict[str, str],
+        source_to_target_columns_map: dict[str, str],
     ) -> None:
         """
         Append the source table rows into a destination table.
@@ -273,8 +426,8 @@ class BaseDatabase(ABC):
         target_table_sqla = self.get_sqla_table(target_table)
         source_table_sqla = self.get_sqla_table(source_table)
 
-        target_columns: List[ColumnClause]
-        source_columns: List[ColumnClause]
+        target_columns: list[ColumnClause]
+        source_columns: list[ColumnClause]
 
         if not source_to_target_columns_map:
             target_columns = [column(col) for col in target_table_sqla.c.keys()]
@@ -297,8 +450,8 @@ class BaseDatabase(ABC):
         self,
         source_table: Table,
         target_table: Table,
-        source_to_target_columns_map: Dict[str, str],
-        target_conflict_columns: List[str],
+        source_to_target_columns_map: dict[str, str],
+        target_conflict_columns: list[str],
         if_conflicts: MergeConflictStrategy = "exception",
     ) -> None:
         """
@@ -320,7 +473,10 @@ class BaseDatabase(ABC):
         :param table: Astro Table to be converted to SQLAlchemy table instance
         """
         return SqlaTable(
-            table.name, table.sqlalchemy_metadata, autoload_with=self.sqlalchemy_engine
+            table.name,
+            table.sqlalchemy_metadata,
+            autoload_with=self.sqlalchemy_engine,
+            extend_existing=True,
         )
 
     # ---------------------------------------------------------
@@ -366,7 +522,7 @@ class BaseDatabase(ABC):
     # Schema Management
     # ---------------------------------------------------------
 
-    def create_schema_if_needed(self, schema: Optional[str]) -> None:
+    def create_schema_if_needed(self, schema: str | None) -> None:
         """
         This function checks if the expected schema exists in the database. If the schema does not exist,
         it will attempt to create it.
@@ -393,7 +549,7 @@ class BaseDatabase(ABC):
 
     def get_sqlalchemy_template_table_identifier_and_parameter(
         self, table: Table, jinja_table_identifier: str
-    ) -> Tuple[str, str]:
+    ) -> tuple[str, str]:
         """
         During the conversion from a Jinja-templated SQL query to a SQLAlchemy query, there is the need to
         convert a Jinja table identifier to a safe SQLAlchemy-compatible table identifier.
@@ -419,3 +575,33 @@ class BaseDatabase(ABC):
             self.get_table_qualified_name(table),
             self.get_table_qualified_name(table),
         )
+
+    def is_native_load_file_available(  # skipcq: PYL-R0201
+        self, source_file: File, target_table: Table  # skipcq: PYL-W0613
+    ) -> bool:
+        """
+        Check if there is an optimised path for source to destination.
+
+        :param source_file: File from which we need to transfer data
+        :param target_table: Table that needs to be populated with file data
+        """
+        return False
+
+    def load_file_to_table_natively(
+        self,
+        source_file: File,
+        target_table: Table,
+        if_exists: LoadExistStrategy = "replace",
+        native_support_kwargs: dict | None = None,
+        **kwargs,
+    ):
+        """
+        Checks if optimised path for transfer between File location to database exists
+        and if it does, it transfers it and returns true else false
+
+        :param source_file: File from which we need to transfer data
+        :param target_table: Table that needs to be populated with file data
+        :param if_exists: Overwrite file if exists. Default False
+        :param native_support_kwargs: kwargs to be used by native loading command
+        """
+        raise NotImplementedError
