@@ -1,10 +1,13 @@
-from typing import List, Optional, Union
+from __future__ import annotations
+
+import io
 
 import pandas as pd
 import smart_open
 
-from astro.constants import FileType
+from astro import constants
 from astro.files.locations import create_file_location
+from astro.files.locations.base import BaseFileLocation
 from astro.files.types import create_file_type
 
 
@@ -18,9 +21,9 @@ class File:
     def __init__(
         self,
         path: str,
-        conn_id: Optional[str] = None,
-        filetype: Union[FileType, None] = None,
-        normalize_config: Optional[dict] = None,
+        conn_id: str | None = None,
+        filetype: constants.FileType | None = None,
+        normalize_config: dict | None = None,
     ):
         """Init file object which represent a single file in local/object stores
 
@@ -29,7 +32,7 @@ class File:
         :param filetype: constant to provide an explicit file type
         :param normalize_config: parameters in dict format of pandas json_normalize() function.
         """
-        self.location = create_file_location(path, conn_id)
+        self.location: BaseFileLocation = create_file_location(path, conn_id)
         self.type = create_file_type(
             path=path, filetype=filetype, normalize_config=normalize_config
         )
@@ -39,7 +42,7 @@ class File:
         return self.location.path
 
     @property
-    def conn_id(self) -> Optional[str]:
+    def conn_id(self) -> str | None:
         return self.location.conn_id
 
     @property
@@ -54,11 +57,11 @@ class File:
 
     def is_binary(self) -> bool:
         """
-        Return a FileType given the filepath. Uses a naive strategy, using the file extension.
+        Return a constants.FileType given the filepath. Uses a naive strategy, using the file extension.
 
         :return: True or False
         """
-        result: bool = self.type.name == FileType.PARQUET
+        result: bool = self.type.name == constants.FileType.PARQUET
         return result
 
     def create_from_dataframe(self, df: pd.DataFrame) -> None:
@@ -71,13 +74,38 @@ class File:
         ) as stream:
             self.type.create_from_dataframe(stream=stream, df=df)
 
-    def export_to_dataframe(self, **kwargs) -> pd.DataFrame:
-        """Read file from all supported location and convert them into dataframes"""
+    def _convert_remote_file_to_byte_stream(self) -> io.IOBase:
+        """
+        Read file from all supported location and convert them into a buffer that can be streamed into other data
+        structures.
+
+        Due to noted issues with using smart_open with pandas (like
+        https://github.com/RaRe-Technologies/smart_open/issues/524), we create a BytesIO or StringIO buffer
+        before exporting to a dataframe. We've found a sizable speed improvement with this optimizat
+
+        Returns: an io object that can be streamed into a dataframe (or other object)
+
+        """
         mode = "rb" if self.is_binary() else "r"
+        remote_obj_buffer = io.BytesIO() if self.is_binary() else io.StringIO()
         with smart_open.open(
             self.path, mode=mode, transport_params=self.location.transport_params
         ) as stream:
-            return self.type.export_to_dataframe(stream, **kwargs)
+            remote_obj_buffer.write(stream.read())
+        remote_obj_buffer.seek(0)
+        return remote_obj_buffer
+
+    def export_to_dataframe(self, **kwargs) -> pd.DataFrame:
+        """Read file from all supported location and convert them into dataframes.
+
+        Due to noted issues with using smart_open with pandas (like
+        https://github.com/RaRe-Technologies/smart_open/issues/524), we create a BytesIO or StringIO buffer
+        before exporting to a dataframe. We've found a sizable speed improvement with this optimization.
+        """
+
+        return self.type.export_to_dataframe(
+            self._convert_remote_file_to_byte_stream(), **kwargs
+        )
 
     def exists(self) -> bool:
         """Check if the file exists or not"""
@@ -90,15 +118,18 @@ class File:
         )
 
     def __str__(self) -> str:
-        return self.location.path
+        return self.location.path  # type: ignore
+
+    def __eq__(self, other):
+        return self.location == other.location and self.type == other.type
 
 
-def get_files(
+def resolve_file_path_pattern(
     path_pattern: str,
-    conn_id: Optional[str] = None,
-    filetype: Union[FileType, None] = None,
-    normalize_config: Optional[dict] = None,
-) -> List[File]:
+    conn_id: str | None = None,
+    filetype: constants.FileType | None = None,
+    normalize_config: dict | None = None,
+) -> list[File]:
     """get file objects by resolving path_pattern from local/object stores
     path_pattern can be
     1. local location - glob pattern
